#!/usr/bin/perl
#
# Modified from Martin Pot's 2003 script at
# http://martybugs.net/linux/hddtemp.cgi
# 
# Graphs motherboard temperatures using mbmon
# rrd_mbtemp.pl

use RRDs;

# define location of rrdtool databases
my $rrd = '/var/lib/rrd';
# define location of images
my $img = '/var/www/rrdtool';

# process data for each temperature sensor (add/delete as required)
# run mbmon -r -c 1 to see what temps your mobo supports, then make
# an educated guess about which component corresponds to which temp
# Note TEMP0 = T 1, TEMP1 = T 2, TEMP2 = T 3
&ProcessHDD("mbtemp", "T 1", "Motherboard");
&ProcessHDD("cputemp", "T 2", "CPU");
&ProcessHDD("ambtemp", "T 3", "Case ambient");

sub ProcessHDD
{
# process HDD
# inputs: $_[0]: component being measured (used for graph filenames)
#          $_[1]: code (mbmon code)
#         $_[2]: description

        # get temp value from mbmon
        my $temp=`/usr/bin/mbmon -c 1 -$_[1]`;
        # remove eol chars and white space
        $temp =~ s/[\n ]//g;
        
        print "$_[2] temp: $temp degrees C\n";

        # if rrdtool database doesn't exist, create it
        if (! -e "$rrd/$_[0].rrd")
        {
                print "creating rrd database for $_[0]...\n";
                RRDs::create "$rrd/$_[0].rrd",
                        "-s 300",
                        "DS:temp:GAUGE:600:0:100",
                        "RRA:AVERAGE:0.5:1:576",
                        "RRA:AVERAGE:0.5:6:672",
                        "RRA:AVERAGE:0.5:24:732",
                        "RRA:AVERAGE:0.5:144:1460";
        }

        # insert value into rrd
        RRDs::update "$rrd/$_[0].rrd",
                "-t", "temp",
                "N:$temp";

        # create graphs
        &CreateGraph($_[0], "day", $_[2]);
        &CreateGraph($_[0], "week", $_[2]);
        &CreateGraph($_[0], "month", $_[2]);
        &CreateGraph($_[0], "year", $_[2]);
}

sub CreateGraph
{
# creates graph
# inputs: $_[0]: component being measured (used for graph filenames)
#         $_[1]: interval (ie, day, week, month, year)
#         $_[2]: description

        RRDs::graph "$img/$_[0]-$_[1].png",
                "--lazy",
                "-s -1$_[1]",
                "--color", "SHADEA#EAE9EE",
                "--color", "SHADEB#EAE9EE",
                "--color", "BACK#295990",
                "-t temperature :: $_[2] ($_[0])",
                "-h", "80", "-w", "600",
                "-a", "PNG",
                "-v degrees C",
                "DEF:temp=$rrd/$_[0].rrd:temp:AVERAGE",
                "LINE2:temp#0000FF:$_[2] ($_[0])",
                "GPRINT:temp:MIN:  Min\\: %2.lf",
                "GPRINT:temp:MAX: Max\\: %2.lf",
                "GPRINT:temp:AVERAGE: Avg\\: %4.1lf",
                "GPRINT:temp:LAST: Current\\: %2.lf degrees C\\n";
        if ($ERROR = RRDs::error) { print "$0: unable to generate $_[0] graph: $ERROR\n"; }
}

