#!/usr/bin/perl

use RRDs;

my $rrdlog = '/var/lib/rrd';
my $graphs = '/data/srv/httpd/htdocs/rrd';

updatememdata ();
updatememgraph ('day');
updatememgraph ('week');
updatememgraph ('month');
updatememgraph ('year');

sub updatememgraph {
        my $period    = $_[0];

        RRDs::graph ("$graphs/memory-$period.png",
                "--start", "-1$period", "-aPNG", "-i", "-z",
                "--alt-y-grid", "-w 600", "-h 100", "-l 0", "-u 100", "-r",
                "--color", "SHADEA#EAE9EE",
                "--color", "SHADEB#EAE9EE",
                "--color", "BACK#EAE9EE",
                "-t memory usage per $period",
                "DEF:used=$rrdlog/mem.rrd:memused:AVERAGE",
                "DEF:free=$rrdlog/mem.rrd:memfree:AVERAGE",
                "DEF:shared=$rrdlog/mem.rrd:memshared:AVERAGE",
                "DEF:buffer=$rrdlog/mem.rrd:membuffers:AVERAGE",
                "DEF:cache=$rrdlog/mem.rrd:memcache:AVERAGE",
                "CDEF:total=used,free,+",
                "CDEF:used2=used,buffer,cache,shared,+,+,-",
                "CDEF:usedpct=100,used2,total,/,*",
                "CDEF:sharedpct=100,shared,total,/,*",
                "CDEF:bufferpct=100,buffer,total,/,*",
                "CDEF:cachepct=100,cache,total,/,*",
                "CDEF:freepct=100,free,total,/,*",
                "AREA:usedpct#0000FF:used memory\\j",
                "STACK:sharedpct#FF0000:shared memory\\j",
                "STACK:bufferpct#FF00FF:buffered memory\\j",
                "STACK:cachepct#FFFF00:cached memory\\j",
                "STACK:freepct#00FF00:free memory\\j",
                "GPRINT:usedpct:MAX:maximal used memory\\:%3.2lf%%",
                "GPRINT:usedpct:AVERAGE:average used memory\\:%3.2lf%%",
                "GPRINT:usedpct:LAST:current used memory\\:%3.2lf%%\\j",
                "GPRINT:sharedpct:MAX:maximal shared memory\\:%3.2lf%%",
                "GPRINT:sharedpct:AVERAGE:average shared memory\\:%3.2lf%%",
                "GPRINT:sharedpct:LAST:current shared memory\\:%3.2lf%%\\j",
                "GPRINT:bufferpct:MAX:maximal buffered memory\\:%3.2lf%%",
                "GPRINT:bufferpct:AVERAGE:average buffered memory\\:%3.2lf%%",
                "GPRINT:bufferpct:LAST:current buffered memory\\:%3.2lf%%\\j",
                "GPRINT:cachepct:MAX:maximal cached memory\\:%3.2lf%%",
                "GPRINT:cachepct:AVERAGE:average cached memory\\:%3.2lf%%",
                "GPRINT:cachepct:LAST:current cached memory\\:%3.2lf%%\\j",
                "GPRINT:freepct:MAX:maximal free memory\\:%3.2lf%%",
                "GPRINT:freepct:AVERAGE:average free memory\\:%3.2lf%%",
                "GPRINT:freepct:LAST:current free memory\\:%3.2lf%%\\j");
        $ERROR = RRDs::error;
        print "Error in RRD::graph for mem: $ERROR\n" if $ERROR;

        RRDs::graph ("$graphs/swap-$period.png",
                "--start", "-1$period", "-aPNG", "-i", "-z",
                "--alt-y-grid", "-w 600", "-h 100", "-l 0", "-u 100", "-r",
                "--color", "SHADEA#EAE9EE",
                "--color", "SHADEB#EAE9EE",
                "--color", "BACK#EAE9EE",
                "-t swap usage per $period",
                "DEF:used=$rrdlog/mem.rrd:swapused:AVERAGE",
                "DEF:free=$rrdlog/mem.rrd:swapfree:AVERAGE",
                "CDEF:total=used,free,+",
                "CDEF:usedpct=100,used,total,/,*",
                "CDEF:freepct=100,free,total,/,*",
                "AREA:usedpct#0000FF:used swap\\j",
                "STACK:freepct#00FF00:free swap\\j",
                "GPRINT:usedpct:MAX:maximal used swap\\:%3.2lf%%",
                "GPRINT:usedpct:AVERAGE:average used swap\\:%3.2lf%%",
                "GPRINT:usedpct:LAST:current used swap\\:%3.2lf%%\\j",
                "GPRINT:freepct:MAX:maximal free swap\\:%3.2lf%%",
                "GPRINT:freepct:AVERAGE:average free swap\\:%3.2lf%%",
                "GPRINT:freepct:LAST:current free swap\\:%3.2lf%%\\j");
        $ERROR = RRDs::error;
        print "Error in RRD::graph for swap: $ERROR\n" if $ERROR;
}

sub updatememdata {
        my ($memused, $memfree, $memshared, $membuffers, $memcache, $swapused, $swapfree);
        if ( ! -e "$rrdlog/mem.rrd") {
                RRDs::create ("$rrdlog/mem.rrd", "--step=300",
                        "DS:memused:ABSOLUTE:600:0:U",
                        "DS:memfree:ABSOLUTE:600:0:U",
                        "DS:memshared:ABSOLUTE:600:0:U",
                        "DS:membuffers:ABSOLUTE:600:0:U",
                        "DS:memcache:ABSOLUTE:600:0:U",
                        "DS:swapused:ABSOLUTE:600:0:U",
                        "DS:swapfree:ABSOLUTE:600:0:U",
                        "RRA:AVERAGE:0.5:1:576",
                        "RRA:AVERAGE:0.5:6:672",
                        "RRA:AVERAGE:0.5:24:732",
                        "RRA:AVERAGE:0.5:144:1460");
                $ERROR = RRDs::error;
                print "Error in RRD::create for mem: $ERROR\n" if $ERROR;
        }

        my @memdata = `free -b -o`;

        my $temp = $memdata[1];

        chomp( $temp );
        my @tempa = split (/\s+/, $temp);
        $memused    = $tempa [2];
        $memfree    = $tempa [3];
        $memshared  = $tempa [4];
        $membuffers = $tempa [5];
        $memcache   = $tempa [6];

        $temp = $memdata[2];
        chomp( $temp );
        @tempa = split (/\s+/, $temp);
        $swapused = $tempa [2];
        $swapfree = $tempa [3];

        print "N:$memused:$memfree:$memshared:$membuffers:$memcache:$swapused:$swapfree\n";

        RRDs::update ("$rrdlog/mem.rrd",
                "-t", "memused:memfree:memshared:membuffers:memcache:swapused:swapfree",
                "N:$memused:$memfree:$memshared:$membuffers:$memcache:$swapused:$swapfree");
        $ERROR = RRDs::error;
        print "Error in RRD::update for mem: $ERROR\n" if $ERROR;
}

