#!/usr/bin/perl
#			Version 1.1
#             rrd_usage.pl  devicename1 [devicename2 ....]     partition usage
#               or
#             rrd_usage.pl  <without parameters>
#                           in this case we get the devicenames from
#		/etc/rrdsettings.cfg
#
#
# extended devicename format:
#
#  <device>:<mountpoint or other useful name>
#
#
#            example: (without extended devicename format)
#             rrd_diskusage.pl  hda1 hda3 hda4
#
#
#            example: (with extended devicename format)
#             rrd_diskusage.pl  hda1:/ hda3:/usr  hda4:blablub
#

use RRDs;

# define location of rrdtool databases
my $rrdMyDatabasePath = '/var/lib/rrd';

# define location of images
my $graphs = '/data/srv/httpd/htdocs/rrd';

my $rrdDatabaseName = '';

# my $rrdDatabasePath;
#print STDERR "$rrdDatabasePath";

#
my @rrdDevs;
if ((scalar @ARGV) > 0) {
  @rrdDevs = @ARGV;
} else {

  # open input file
  open(inFH, "< /etc/rrdsettings.cfg")  or usage();
  while (defined (my $inLine = <inFH>)) {
    if ($inLine =~/^DISKDEV=/o) {
      (my $devname = $inLine) =~ s/^DISKDEV=//og;
      chomp $devname;
      push(@rrdDevs, $devname);
    }
  }

  close(inFH);
}

# to avoid a lot of emails from cron when not configured
if (exists $ENV{'RRD_SILENT'}) {
  exit 0  if ((scalar @rrdDevs) == 0);
} else {
  usage() if ((scalar @rrdDevs) == 0);
}

#..............................................................................
# fill the databases
fill_rrdSIZE ();

#..............................................................................
sub create_rrdSIZE($) {
  my $rrdFile = $_[0];
  # awaiting an update every 300 secs
  my $rrdStep = 300;
  # data source = ...GAUGE, max 600 sec wait before UNKNOWN,
  #   no min, no max
  # 0.5:1: average value calc. with 1 entry = 300sec
  # 0.5:5: average value calc. with 5 entries = 5*300sec
  # h36    = 3600*36     => 129600sec/300      => 432
  # d7a2   = 3600*24*7   => 604800sec/300 /2   => 1008; 10min average
  # d30a6  = 3600*24*30  => 2592000sec/300 /6  => 1440; 30min average
  # d365a12= 3600*24*365 => 31536000sec/300 /12=> 8760; 1h    average
  # d7a6   = 3600*24*7   => 604800sec/300 /6   => 336;  30min max
  # d30a12 = 3600*24*30  => 2592000sec/300 /12 => 720;  1h    max
  # d365a24= 3600*24*365 => 31536000sec/300 /24=> 4380; 2h    max

  RRDs::create("$rrdFile",
	       '--step', $rrdStep,
	       'DS:used:GAUGE:'     .($rrdStep*2).':0:U',
	       'DS:avail:GAUGE:' .($rrdStep*2).':0:U',
	       'RRA:AVERAGE:0.5:1:432',
	       'RRA:AVERAGE:0.5:2:1008',
	       'RRA:AVERAGE:0.5:6:1440',
	       'RRA:AVERAGE:0.5:12:8760',
	       'RRA:MAX:0.5:6:336',
	       'RRA:MAX:0.5:12:720',
	       'RRA:MAX:0.5:24:4380'
	      );
	my $ERR = RRDs::error;
	die "ERROR while creating $rrdFile: $ERR\n" if $ERR;
}

#..............................................................................
sub create_rrdACCESS($) {
  my $rrdFile = $_[0];
  # awaiting an update every 300 secs
  my $rrdStep = 300;
                RRDs::create ("$rrdFile",
	       '--step', $rrdStep,
           "DS:readsect:COUNTER:600:0:5000000000",
           "DS:writesect:COUNTER:600:0:5000000000",
           "RRA:AVERAGE:0.5:1:576",
           "RRA:AVERAGE:0.5:6:672",
           "RRA:AVERAGE:0.5:24:732",
           "RRA:AVERAGE:0.5:144:1460"	      
           );
	my $ERR = RRDs::error;
	die "ERROR while creating $rrdFile: $ERR\n" if $ERR;
}

#..............................................................................
sub update_rrdSIZE ($$$$$){

  my $rrdFile	= $_[0];
  #my $dataTime = $_[1];
  #my $used     = $_[2];
  #my $avail    = $_[3];
  my $device	= $_[4];

 updategraphSIZE ('day',$rrdFile,$device);
 updategraphSIZE ('week',$rrdFile,$device);
 updategraphSIZE ('month',$rrdFile,$device);
 updategraphSIZE ('year',$rrdFile,$device);

  RRDs::update("$rrdFile", $_[1].':'.$_[2].':'.$_[3]);
  my $ERR = RRDs::error;
  die "ERROR while updating $rrdFile: $ERR\n" if $ERR;
}

#..............................................................................
sub fill_rrdSIZE () {
  # extract the data from df
  my $rrdTime = time;
  my @dfLines = `df -P`;

  foreach my $dfLine (@dfLines) {
    chomp $dfLine;
    foreach my $rrdDev (@rrdDevs) {
      my $realDev ='';
      my $extDev  ='';
      my $dataDev ='';
      
      if ($rrdDev =~ /^(.+)\:(.+)/) {
	$realDev  = $1;
	$extDev   = ''. $2;
	$extDev   =~ s:/:\,:og;
	$dataDev  = $realDev;
	$dataDev  =~ s/\/dev\///;
	$dataDev  =~ s/mapper\///;
	$dataDev  =~ s/\/\/192.168.1.5\/Volume_1\///;

#	print "realDev - \t".$realDev."\n" ;
#	print "extDev - \t".$extDev."\n" ;
#	print "dataDev - \t".$dataDev."\n" ;

      } else {
	 $realDev  = $rrdDev;
      }

      my $rrdFile  = "$rrdMyDatabasePath" . "/usage_Size_" . $dataDev . "." . "rrd";

 	
      if ($dfLine =~ /$realDev\s+\d+\s+(\d+)+\s+(\d+)/g ) {
      	my $usedM = sprintf("%d",$1/1024);
	my $availM = sprintf("%d",$2/1024);

	print "availM - \t".$availM."\n" ;
	print "rrdFile - \t".$rrdFile."\n\n" ;
	
	# fill database
	create_rrdSIZE($rrdFile) if not -w $rrdFile;
	update_rrdSIZE($rrdFile,
		   $rrdTime,
		   $usedM,
		   $availM,
		   $dataDev)  if -w $rrdFile;
      }
        
    }
  }
}

#..............................................................................

sub updategraphSIZE {
        my $period	= $_[0];
        my $rrdfile	= $_[1];
        my $realDev	= $_[2];
        my $shifter	= "0" ;
        my $shifterL	= "" ;
        		
		if ( $period eq "day" ) {
			$shifter = 86400 ;
			$shifterL = "1 Day" ;
		}
		
		if ( $period eq "week" ) {
			$shifter = 604800 ;
			$shifterL = "7 Day";
		}
		
		if ( $period eq "month" ) {
			$shifter = 1209600 ;
			$shifterL = "14 Day" ;
		}
		
		if ( $period eq "year" ) {
			$shifter = 4838400 ;
			$shifterL = "8 Week" ;
		}

						
        RRDs::graph ("$graphs/usage-Size-$realDev-$period.png",
                "--start", "-1$period", "-aPNG", "-i", "-z", "-r",
                "--alt-y-grid", "-w 600", "-h 100", "-l 0",
                "--color", "SHADEA#EAE9EE",
                "--color", "SHADEB#EAE9EE",
                "--color", "BACK#EAE9EE",
				"--vertical-label", "Partition Size",
                "-t $realDev disk usage - $period",
                "DEF:used=$rrdfile:used:AVERAGE",
                "DEF:avail=$rrdfile:avail:AVERAGE",
                "DEF:oldused=$rrdfile:used:AVERAGE:start=-1$period-$shifter",
                "SHIFT:oldused:$shifter",
				"CDEF:MBused=used,1048576,*",
				"CDEF:MBavail=avail,1048576,*",
				"CDEF:total=MBused,MBavail,+",
				"CDEF:wused=oldused,1048576,*",
                "AREA:total#32CD32:Free",
                "GPRINT:MBavail:MAX:  Max\\: %3.2lf%Sb",
                "GPRINT:MBavail:AVERAGE: Avg\\: %3.2lf%Sb",
                "GPRINT:MBavail:LAST: Current\\: %3.2lf%Sb\\n",
                "AREA:MBused#4169E1:Used",
                "LINE1:MBused#0033CC",
                "GPRINT:MBused:MAX:  Max\\: %3.2lf%Sb",
                "GPRINT:MBused:AVERAGE: Avg\\: %3.2lf%Sb",
                "GPRINT:MBused:LAST: Current\\: %3.2lf%Sb\\n",
                "LINE1:wused#FF33CC:$shifterL",
                "GPRINT:wused:MAX:  Max\\: %3.2lf%Sb",
                "GPRINT:wused:AVERAGE: Avg\\: %3.2lf%Sb",
                "GPRINT:wused:LAST: Current\\: %3.2lf%Sb\\n",
                "HRULE:0#000000");   
       	        $ERROR = RRDs::error;
                print "Error in RRD::graph for disk: $ERROR\n" if $ERROR;

}

sub updategraphACCESS {
        my $period	= $_[0];
        my $rrdfile	= $_[1];
        my $realDev	= $_[2];

        RRDs::graph ("$graphs/usage-Access-$realDev-$period.png",
                "--start", "-1$period", "-aPNG", "-i", "-z",
                "--alt-y-grid", "-w 600", "-h 200", "-l 0", "-r",
                "--color", "SHADEA#EAE9EE",
                "--color", "SHADEB#EAE9EE",
                "--color", "BACK#EAE9EE",
                "-t disk access per $period",
                "DEF:read=$rrdlog/disk.rrd:readsect:AVERAGE",
                "DEF:write=$rrdlog/disk.rrd:writesect:AVERAGE",
                "AREA:read#0000FF:sectors read from disk per second\\j",
                "STACK:write#00FF00:sectors written to disk per second\\j",
                "GPRINT:read:MAX:maximal read sectors\\:%8.0lf",
                "GPRINT:read:AVERAGE:average read sectors\\:%8.0lf",
                "GPRINT:read:LAST:current read sectors\\:%8.0lf\\j",
                "GPRINT:write:MAX:maximal written sectors\\:%8.0lf",
                "GPRINT:write:AVERAGE:average written sectors\\:%8.0lf",
                "GPRINT:write:LAST:current written sectors\\:%8.0lf\\j");
        $ERROR = RRDs::error;
        print "Error in RRD::graph for disk: $ERROR\n" if $ERROR;
}
